@@ -25,6 +25,7 @@
 #include <errno.h>
 #include <getopt.h>
 #include <libgen.h>
+#include <pthread.h>
 #include <signal.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -100,7 +101,6 @@ static struct arguments my_args = {
 static void prevent_forking(void)
 {
 	FILE *f;
-	int fd = -1;
 	size_t len = 0;
 	char *line = NULL;
 	char path[MAXPATHLEN];
@@ -110,7 +110,7 @@ static void prevent_forking(void)
 		return;
 
 	while (getline(&line, &len, f) != -1) {
-		int ret;
+		int fd, ret;
 		char *p, *p2;
 
 		p = strchr(line, ':');
@@ -146,11 +146,11 @@ static void prevent_forking(void)
 			goto on_error;
 		}
 
-		if (write(fd, "1", 1) != 1)
+		ret = write(fd, "1", 1);
+		if (ret != 1)
 			SYSERROR("Failed to write to \"%s\"", path);
 
 		close(fd);
-		fd = -1;
 		break;
 	}
 
@@ -267,7 +267,7 @@ int main(int argc, char *argv[])
 	if (ret < 0)
 		exit(EXIT_FAILURE);
 
-	ret = sigprocmask(SIG_SETMASK, &mask, &omask);
+	ret = pthread_sigmask(SIG_SETMASK, &mask, &omask);
 	if (ret < 0)
 		exit(EXIT_FAILURE);
 
@@ -328,6 +328,11 @@ int main(int argc, char *argv[])
 		/* restore default signal handlers */
 		for (i = 1; i < NSIG; i++) {
 			sighandler_t sigerr;
+
+			if (i == SIGILL || i == SIGSEGV || i == SIGBUS ||
+			    i == SIGSTOP || i == SIGKILL || i == 32 || i == 33)
+				continue;
+
 			sigerr = signal(i, SIG_DFL);
 			if (sigerr == SIG_ERR) {
 				DEBUG("%s - Failed to reset to default action "
@@ -336,7 +341,7 @@ int main(int argc, char *argv[])
 			}
 		}
 
-		ret = sigprocmask(SIG_SETMASK, &omask, NULL);
+		ret = pthread_sigmask(SIG_SETMASK, &omask, NULL);
 		if (ret < 0) {
 			SYSERROR("Failed to set signal mask");
 			exit(EXIT_FAILURE);
@@ -364,7 +369,7 @@ int main(int argc, char *argv[])
 	if (ret < 0)
 		exit(EXIT_FAILURE);
 
-	ret = sigprocmask(SIG_SETMASK, &omask, NULL);
+	ret = pthread_sigmask(SIG_SETMASK, &omask, NULL);
 	if (ret < 0) {
 		SYSERROR("Failed to set signal mask");
 		exit(EXIT_FAILURE);
