@@ -29,8 +29,8 @@ static char *nop_process_label_get(pid_t pid)
 	return NULL;
 }
 
-static int nop_process_label_set(const char *label, int use_default,
-				 int on_exec)
+static int nop_process_label_set(const char *label, struct lxc_conf *conf,
+				 bool use_default, bool on_exec)
 {
 	return 0;
 }
