@@ -27,23 +27,24 @@
 #include "conf.h"
 
 #ifdef HAVE_SECCOMP
-int lxc_seccomp_load(struct lxc_conf *conf);
-int lxc_read_seccomp_config(struct lxc_conf *conf);
-void lxc_seccomp_free(struct lxc_conf *conf);
+extern int lxc_seccomp_load(struct lxc_conf *conf);
+extern int lxc_read_seccomp_config(struct lxc_conf *conf);
+extern void lxc_seccomp_free(struct lxc_conf *conf);
 #else
-static inline int lxc_seccomp_load(struct lxc_conf *conf) {
+static inline int lxc_seccomp_load(struct lxc_conf *conf)
+{
 	return 0;
 }
 
-static inline int lxc_read_seccomp_config(struct lxc_conf *conf) {
+static inline int lxc_read_seccomp_config(struct lxc_conf *conf)
+{
 	return 0;
 }
 
-static inline void lxc_seccomp_free(struct lxc_conf *conf) {
-	if (conf->seccomp) {
-		free(conf->seccomp);
-		conf->seccomp = NULL;
-	}
+static inline void lxc_seccomp_free(struct lxc_conf *conf)
+{
+	free(conf->seccomp);
+	conf->seccomp = NULL;
 }
 #endif
 
