@@ -27,6 +27,10 @@
 #include <stdint.h>
 #include "list.h"
 
+#define LXC_MAINLOOP_ERROR -1
+#define LXC_MAINLOOP_CONTINUE 0
+#define LXC_MAINLOOP_CLOSE 1
+
 struct lxc_epoll_descr {
 	int epfd;
 	struct lxc_list handlers;
