@@ -163,18 +163,18 @@ int main(int argc, char *argv[])
 	if (lxc_arguments_parse(&my_args, argc, argv))
 		exit(ret);
 
-	if (!my_args.log_file)
-		my_args.log_file = "none";
+	/* Only create log if explicitly instructed */
+	if (my_args.log_file || my_args.log_priority) {
+		log.name = my_args.name;
+		log.file = my_args.log_file;
+		log.level = my_args.log_priority;
+		log.prefix = my_args.progname;
+		log.quiet = my_args.quiet;
+		log.lxcpath = my_args.lxcpath[0];
 
-	log.name = my_args.name;
-	log.file = my_args.log_file;
-	log.level = my_args.log_priority;
-	log.prefix = my_args.progname;
-	log.quiet = my_args.quiet;
-	log.lxcpath = my_args.lxcpath[0];
-
-	if (lxc_log_init(&log))
-		exit(ret);
+		if (lxc_log_init(&log))
+			exit(ret);
+	}
 
 	if (geteuid()) {
 		if (access(my_args.lxcpath[0], O_RDONLY) < 0) {
@@ -745,7 +745,7 @@ static char *mount_tmpfs(const char *oldname, const char *newname,
 	int ret, fd;
 	size_t len;
 	char *premount = NULL;
-	FILE *fp;
+	FILE *fp = NULL;
 
 	if (arg->tmpfs && arg->keepdata) {
 		fprintf(stderr, "%s\n",
@@ -810,7 +810,7 @@ static char *mount_tmpfs(const char *oldname, const char *newname,
 err_close:
 	if (fd > 0)
 		close(fd);
-	else
+	else if (fp)
 		fclose(fp);
 err_free:
 	free(premount);
