@@ -155,6 +155,28 @@ static int set_and_clear_complete_netdev(struct lxc_container *c)
 	return 0;
 }
 
+static int set_invalid_netdev(struct lxc_container *c) {
+	if (c->set_config_item(c, "lxc.net.0.asdf", "veth")) {
+		lxc_error("%s\n", "lxc.net.0.asdf should be invalid");
+		return -1;
+	}
+
+	if (c->set_config_item(c, "lxc.net.2147483647.type", "veth")) {
+		lxc_error("%s\n", "lxc.net.2147483647.type should be invalid");
+		return -1;
+	}
+
+	if (c->set_config_item(c, "lxc.net.0.", "veth")) {
+		lxc_error("%s\n", "lxc.net.0. should be invalid");
+		return -1;
+	}
+
+	c->clear_config(c);
+	c->lxc_conf = NULL;
+
+	return 0;
+}
+
 int test_idmap_parser(void)
 {
 	size_t i;
@@ -274,16 +296,16 @@ static int set_get_compare_clear_save_load_network(
 
 int main(int argc, char *argv[])
 {
+	int ret;
 	struct lxc_container *c;
-	int fd = -1;
-	int ret = EXIT_FAILURE;
+	int fd = -1, fret = EXIT_FAILURE;
 	char tmpf[] = "lxc-parse-config-file-XXXXXX";
 	char retval[4096] = {0};
 
 	fd = mkstemp(tmpf);
 	if (fd < 0) {
 		lxc_error("%s\n", "Could not create temporary file");
-		exit(ret);
+		exit(fret);
 	}
 	close(fd);
 
@@ -303,9 +325,9 @@ int main(int argc, char *argv[])
 	/* REMOVE IN LXC 3.0
 	   legacy ps keys
 	 */
-	if (set_get_compare_clear_save_load(c, "lxc.pts", "1000", tmpf, true) <
+	if (set_get_compare_clear_save_load(c, "lxc.pty.max", "1000", tmpf, true) <
 	    0) {
-		lxc_error("%s\n", "lxc.pts");
+		lxc_error("%s\n", "lxc.pty.max");
 		goto non_test_error;
 	}
 
@@ -316,15 +338,6 @@ int main(int argc, char *argv[])
 		goto non_test_error;
 	}
 
-	/* REMOVE IN LXC 3.0
-	   legacy tty.max keys
-	 */
-	if (set_get_compare_clear_save_load(c, "lxc.tty", "4", tmpf, true) <
-	    0) {
-		lxc_error("%s\n", "lxc.tty");
-		goto non_test_error;
-	}
-
 	/* lxc.tty.max */
 	if (set_get_compare_clear_save_load(c, "lxc.tty.max", "4", tmpf, true) <
 	    0) {
@@ -332,15 +345,6 @@ int main(int argc, char *argv[])
 		goto non_test_error;
 	}
 
-	/* REMOVE IN LXC 3.0
-	   legacy devttydir keys
-	 */
-	if (set_get_compare_clear_save_load(c, "lxc.devttydir", "not-dev", tmpf,
-					    true) < 0) {
-		lxc_error("%s\n", "lxc.devttydir");
-		goto non_test_error;
-	}
-
 	/* lxc.tty.dir */
 	if (set_get_compare_clear_save_load(c, "lxc.tty.dir", "not-dev", tmpf,
 					    true) < 0) {
@@ -348,33 +352,6 @@ int main(int argc, char *argv[])
 		goto non_test_error;
 	}
 
-	/* REMOVE IN LXC 3.0
-	   legacy security keys
-	 */
-	if (set_get_compare_clear_save_load(c, "lxc.aa_profile", "unconfined",
-					    tmpf, true) < 0) {
-		lxc_error("%s\n", "lxc.aa_profile");
-		goto non_test_error;
-	}
-
-	/* REMOVE IN LXC 3.0
-	   legacy security keys
-	 */
-	if (set_get_compare_clear_save_load(c, "lxc.aa_allow_incomplete", "1",
-					    tmpf, true) < 0) {
-		lxc_error("%s\n", "lxc.aa_allow_incomplete");
-		goto non_test_error;
-	}
-
-	/* REMOVE IN LXC 3.0
-	   legacy security keys
-	 */
-	if (set_get_compare_clear_save_load(c, "lxc.se_context", "system_u:system_r:lxc_t:s0:c22",
-					    tmpf, true) < 0) {
-		lxc_error("%s\n", "lxc.se_context");
-		goto non_test_error;
-	}
-
 	/* lxc.apparmor.profile */
 	if (set_get_compare_clear_save_load(c, "lxc.apparmor.profile", "unconfined",
 					    tmpf, true) < 0) {
@@ -427,34 +404,6 @@ int main(int argc, char *argv[])
 	c->clear_config(c);
 	c->lxc_conf = NULL;
 
-	/* lxc.id_map
-	 * We can't really save the config here since save_config() wants to
-	 * chown the container's directory but we haven't created an on-disk
-	 * container. So let's test set-get-clear.
-	 */
-	if (set_get_compare_clear_save_load(
-		c, "lxc.id_map", "u 0 100000 1000000000", NULL, false) < 0) {
-		lxc_error("%s\n", "lxc.id_map");
-		goto non_test_error;
-	}
-
-	if (!c->set_config_item(c, "lxc.id_map", "u 1 100000 10000000")) {
-		lxc_error("%s\n", "failed to set config item "
-				  "\"lxc.id_map\" to \"u 1 100000 10000000\"");
-		return -1;
-	}
-
-	if (!c->set_config_item(c, "lxc.id_map", "g 1 100000 10000000")) {
-		lxc_error("%s\n", "failed to set config item "
-				  "\"lxc.id_map\" to \"g 1 100000 10000000\"");
-		return -1;
-	}
-
-	if (!c->get_config_item(c, "lxc.id_map", retval, sizeof(retval))) {
-		lxc_error("%s\n", "failed to get config item \"lxc.cgroup\"");
-		return -1;
-	}
-
 	/* lxc.idmap
 	 * We can't really save the config here since save_config() wants to
 	 * chown the container's directory but we haven't created an on-disk
@@ -479,32 +428,13 @@ int main(int argc, char *argv[])
 	}
 
 	if (!c->get_config_item(c, "lxc.idmap", retval, sizeof(retval))) {
-		lxc_error("%s\n", "failed to get config item \"lxc.cgroup\"");
+		lxc_error("%s\n", "failed to get config item \"lxc.idmap\"");
 		return -1;
 	}
 
 	c->clear_config(c);
 	c->lxc_conf = NULL;
 
-	/* REMOVE IN LXC 3.0
-	   legacy lxc.loglevel key
-	 */
-	if (set_get_compare_clear_save_load(c, "lxc.loglevel", "DEBUG", tmpf,
-					    true) < 0) {
-		lxc_error("%s\n", "lxc.loglevel");
-		goto non_test_error;
-	}
-
-	/* REMOVE IN LXC 3.0
-	   legacy lxc.logfile key
-	 */
-	if (set_get_compare_clear_save_load(c, "lxc.logfile", "/some/path",
-					    tmpf, true) < 0) {
-		lxc_error("%s\n", "lxc.logfile");
-		goto non_test_error;
-	}
-
-
 	/* lxc.log.level */
 	if (set_get_compare_clear_save_load(c, "lxc.log.level", "DEBUG", tmpf,
 					    true) < 0) {
@@ -519,15 +449,6 @@ int main(int argc, char *argv[])
 		goto non_test_error;
 	}
 
-	/* REMOVE IN LXC 3.0
-	   legacy lxc.mount key
-	 */
-	if (set_get_compare_clear_save_load(c, "lxc.mount", "/some/path", NULL,
-					    true) < 0) {
-		lxc_error("%s\n", "lxc.mount");
-		goto non_test_error;
-	}
-
 	/* lxc.mount.fstab */
 	if (set_get_compare_clear_save_load(c, "lxc.mount.fstab", "/some/path", NULL,
 					    true) < 0) {
@@ -558,15 +479,6 @@ int main(int argc, char *argv[])
 		goto non_test_error;
 	}
 
-	/* REMOVE IN LXC 3.0
-	   legacy lxc.rootfs key
-	 */
-	if (set_get_compare_clear_save_load(c, "lxc.rootfs", "/some/path", tmpf,
-					    true) < 0) {
-		lxc_error("%s\n", "lxc.rootfs");
-		goto non_test_error;
-	}
-
 	/* lxc.rootfs.path */
 	if (set_get_compare_clear_save_load(c, "lxc.rootfs.path", "/some/path", tmpf,
 					    true) < 0) {
@@ -588,15 +500,6 @@ int main(int argc, char *argv[])
 		goto non_test_error;
 	}
 
-	/* REMOVE IN LXC 3.0
-	   legacy lxc.utsname key
-	 */
-	if (set_get_compare_clear_save_load(c, "lxc.utsname", "the-shire", tmpf,
-					    true) < 0) {
-		lxc_error("%s\n", "lxc.utsname");
-		goto non_test_error;
-	}
-
 	/* lxc.uts.name */
 	if (set_get_compare_clear_save_load(c, "lxc.uts.name", "the-shire", tmpf,
 					    true) < 0) {
@@ -683,15 +586,6 @@ int main(int argc, char *argv[])
 		goto non_test_error;
 	}
 
-	/* REMOVE IN LXC 3.0
-	   legacy lxc.console key
-	 */
-	if (set_get_compare_clear_save_load(c, "lxc.console", "none", tmpf,
-					    true) < 0) {
-		lxc_error("%s\n", "lxc.console");
-		goto non_test_error;
-	}
-
 	/* lxc.console.path */
 	if (set_get_compare_clear_save_load(c, "lxc.console.path", "none", tmpf,
 					    true) < 0) {
@@ -706,15 +600,6 @@ int main(int argc, char *argv[])
 		goto non_test_error;
 	}
 
-	/* REMOVE IN LXC 3.0
-	   legacy seccomp key
-	 */
-	if (set_get_compare_clear_save_load(
-		c, "lxc.seccomp", "/some/seccomp/file", tmpf, true) < 0) {
-		lxc_error("%s\n", "lxc.seccomp");
-		goto non_test_error;
-	}
-
 	/* lxc.seccomp.profile */
 	if (set_get_compare_clear_save_load(
 		c, "lxc.seccomp.profile", "/some/seccomp/file", tmpf, true) < 0) {
@@ -729,15 +614,6 @@ int main(int argc, char *argv[])
 		goto non_test_error;
 	}
 
-	/* REMOVE IN LXC 3.0
-	   legacy lxc.haltsignal key
-	 */
-	if (set_get_compare_clear_save_load(c, "lxc.haltsignal", "1", tmpf,
-					    true) < 0) {
-		lxc_error("%s\n", "lxc.haltsignal");
-		goto non_test_error;
-	}
-
 	/* lxc.signal.halt */
 	if (set_get_compare_clear_save_load(c, "lxc.signal.halt", "1", tmpf,
 					    true) < 0) {
@@ -745,15 +621,6 @@ int main(int argc, char *argv[])
 		goto non_test_error;
 	}
 
-	/* REMOVE IN LXC 3.0
-	   legacy lxc.rebootsignal key
-	 */
-	if (set_get_compare_clear_save_load(c, "lxc.rebootsignal", "1", tmpf,
-					    true) < 0) {
-		lxc_error("%s\n", "lxc.rebootsignal");
-		goto non_test_error;
-	}
-
 	/* lxc.signal.reboot */
 	if (set_get_compare_clear_save_load(c, "lxc.signal.reboot", "1", tmpf,
 					    true) < 0) {
@@ -761,15 +628,6 @@ int main(int argc, char *argv[])
 		goto non_test_error;
 	}
 
-	/* REMOVE IN LXC 3.0
-	   legacy lxc.stopsignal key
-	 */
-	if (set_get_compare_clear_save_load(c, "lxc.stopsignal", "1", tmpf,
-					    true) < 0) {
-		lxc_error("%s\n", "lxc.stopsignal");
-		goto non_test_error;
-	}
-
 	/* lxc.signal.stop */
 	if (set_get_compare_clear_save_load(c, "lxc.signal.stop", "1", tmpf,
 					    true) < 0) {
@@ -805,13 +663,6 @@ int main(int argc, char *argv[])
 		goto non_test_error;
 	}
 
-	/* lxc.utsname */
-	if (set_get_compare_clear_save_load(c, "lxc.utsname", "get-schwifty",
-					    tmpf, true) < 0) {
-		lxc_error("%s\n", "lxc.utsname");
-		goto non_test_error;
-	}
-
 	/* lxc.monitor.unshare */
 	if (set_get_compare_clear_save_load(c, "lxc.monitor.unshare", "1", tmpf,
 					    true) < 0) {
@@ -833,15 +684,6 @@ int main(int argc, char *argv[])
 		goto non_test_error;
 	}
 
-	/* REMOVE IN LXC 3.0
-	   legacy lxc.init_cmd key
-	 */
-	if (set_get_compare_clear_save_load(c, "lxc.init_cmd", "/bin/bash",
-					    tmpf, true) < 0) {
-		lxc_error("%s\n", "lxc.init_cmd");
-		goto non_test_error;
-	}
-
 	/* lxc.init.cmd */
 	if (set_get_compare_clear_save_load(c, "lxc.init.cmd", "/bin/bash",
 					    tmpf, true) < 0) {
@@ -849,15 +691,6 @@ int main(int argc, char *argv[])
 		goto non_test_error;
 	}
 
-	/* REMOVE IN LXC 3.0
-	   legacy lxc.init_uid key
-	 */
-	if (set_get_compare_clear_save_load(c, "lxc.init_uid", "1000", tmpf,
-					    true) < 0) {
-		lxc_error("%s\n", "lxc.init_uid");
-		goto non_test_error;
-	}
-
 	/* lxc.init.uid */
 	if (set_get_compare_clear_save_load(c, "lxc.init.uid", "1000", tmpf,
 					    true) < 0) {
@@ -865,15 +698,6 @@ int main(int argc, char *argv[])
 		goto non_test_error;
 	}
 
-	/* REMOVE IN LXC 3.0
-	   legacy lxc.init_gid key
-	 */
-	if (set_get_compare_clear_save_load(c, "lxc.init_gid", "1000", tmpf,
-					    true) < 0) {
-		lxc_error("%s\n", "lxc.init_gid");
-		goto non_test_error;
-	}
-
 	/* lxc.init.gid */
 	if (set_get_compare_clear_save_load(c, "lxc.init.gid", "1000", tmpf,
 					    true) < 0) {
@@ -895,12 +719,17 @@ int main(int argc, char *argv[])
 		goto non_test_error;
 	}
 
-	/* REMOVE IN LXC 3.0
-	   legacy lxc.limit.* key
-	 */
-	if (set_get_compare_clear_save_load(c, "lxc.limit.nofile", "65536",
-					    tmpf, true) < 0) {
-		lxc_error("%s\n", "lxc.limit.nofile");
+	/* lxc.sysctl */
+	if (set_get_compare_clear_save_load(c, "lxc.sysctl.net.core.somaxconn", "256", tmpf,
+					    true) < 0) {
+		lxc_error("%s\n", "lxc.sysctl.net.core.somaxconn");
+		goto non_test_error;
+	}
+
+	/* lxc.proc */
+	if (set_get_compare_clear_save_load(c, "lxc.proc.oom_score_adj", "10", tmpf,
+					    true) < 0) {
+		lxc_error("%s\n", "lxc.proc.oom_score_adj");
 		goto non_test_error;
 	}
 
@@ -970,12 +799,6 @@ int main(int argc, char *argv[])
 		goto non_test_error;
 	}
 
-	if (!set_get_compare_clear_save_load(c, "lxc.net.0.asdf", "veth",
-					    tmpf, true)) {
-		lxc_error("%s\n", "lxc.net.0.asdf");
-		goto non_test_error;
-	}
-
 	if (set_get_compare_clear_save_load_network(
 		c, "lxc.net.0.macvlan.mode", "private", tmpf, true,
 		"macvlan")) {
@@ -1070,10 +893,28 @@ int main(int argc, char *argv[])
 		goto non_test_error;
 	}
 
-	ret = EXIT_SUCCESS;
+	if (set_invalid_netdev(c) < 0) {
+		lxc_error("%s\n", "failed to reject invalid configuration");
+		goto non_test_error;
+	}
+
+	ret = set_get_compare_clear_save_load(c, "lxc.hook.version", "1", tmpf, true);
+	if (ret < 0) {
+		lxc_error("%s\n", "lxc.hook.version");
+		goto non_test_error;
+	}
+
+	ret = set_get_compare_clear_save_load(c, "lxc.hook.version", "2", tmpf, true);
+	if (ret == 0) {
+		lxc_error("%s\n", "lxc.hook.version");
+		goto non_test_error;
+	}
+
+	fret = EXIT_SUCCESS;
+
 non_test_error:
 	(void)unlink(tmpf);
 	(void)rmdir(dirname(c->configfile));
 	lxc_container_put(c);
-	exit(ret);
+	exit(fret);
 }
